#!/usr/local/bin/perl
=head1 スクリプト名

wemo_r2.cgi - メモ検索（Wemo システムの CGI の一つ）

=head2 概要

メモディレクトリのメモファイルを正規表現パターンで検索する。

=cut
use CGI qw(:cgi);
use URI::Escape;

$docroot = "/homepage";# HTTPサーバーのドキュメント格納ディレクトリ
$cgidir = "/cgi-bin";#$ENV{'CGIDIR'};
$memodir = "memo";# メモファイル格納ディレクトリの相対パス
$filepat = "\\d{4}-\\d{2}-\\d{2}T\\d{2}-\\d{2}-\\d{2}\\.txt";# メモファイル名の正規表現パターン

# メモのカテゴリのASCII文字列をキーに日本語文字列を連想配列の値として取得
# メモ格納ディレクトリにある category.txt から読み込む
if(open(IN, "<$docroot/$memodir/category_utf-8.txt")){
	while(<IN>){
		chomp;
		($catstr, $category) = split(/\t/);
		$categories{$catstr} = $category;
	}
	close(IN);
}else{
	# category.txt が無ければ、次の連想配列のデータを使う
	%categories = (
		"favor" => "お気に入り",
		"diary" => "日記",
		"memo" => "メモ",
		"sched" => "スケジュール",
		"annot" => "註釈",
		"file" => "マイドキュメント",
	);
}

# フォームからの入力を取得
if (param('pattern')) {
	$pattern = param('pattern');
}elsif($ENV{'PATH_INFO'}){
	($safepattern = $ENV{'PATH_INFO'}) =~ s/^\/(.+)$/$1/;
	$pattern = uri_unescape($safepattern);
}else{
	$pattern = ".";
}

if($pattern =~ /[<>]/){
	$pattern = "不正な検索文字列です。";
}

# メモ検索結果の CGI 先頭部分出力
print <<HEADER;
Content-type: text/html; charset=UTF-8

<HTML>
<HEAD>
<TITLE>Wemo Retrieve Results</TITLE>
<META HTTP-EQUIV="Content-Type" content="text/html; charset=UTF-8">
<link rel="stylesheet" type="text/css" href="/mystyle.css">
</HEAD>
<BODY>
<form method="POST" action="$cgidir/wemo_r2.cgi" target="submain">
Retrieval string: <input type="text" name="pattern">
<input type="submit">
<input type="reset">
</form>
<div class="emph">Wemo Retrieve Results</div>
Retieval string：$pattern<br>
<table border>
<tr><th>Date</th><th>Categ.</th><th>Title</th><th>Update</th></tr>
HEADER

# メモディレクトリのメモファイル名の取得
opendir(DIR, "$docroot/$memodir");
@memo = grep(/^$filepat$/i, readdir(DIR));
closedir(DIR);

# メモ検索
foreach $memo (sort @memo){
	open(IN, "<$docroot/$memodir/$memo");# メモファイルをオープンする
	while(<IN>){
		# メモの日付から、yyyy-mm-dd 形式の日付を得る
		if(/^(\d{4}-\d{2}-\d{2}T\d{2}:\d{2}:\d{2} \(\w{3}\.\))$/i){
			$date = $1;
		# メモのタイトルとアンカーの NAME属性からカテゴリと属性値を取得
		}elsif(/^C: (.+)$/i){
			# カテゴリを取得
			$category = $1;
		}elsif(/^T: (.+)$/i){
			# タイトルを取得
			$title = $1;
		}elsif(/^Up*da*t*e*: (.+)$/i){
			# 更新日付取得
			$update = $1;
		}
		# パターンにマッチすれば、
		if(/$pattern/i){
			$match++;
		}
	}
	close(IN);
	if($match > 0){
		print "<tr><td><a href=\"$cgidir/wemo.cgi/$memo\">$date</a></td><td>$categories{$category}</td><td>$title</td><td>$update</td></tr>\n" ;
		$count++;
	}
	$date = "";$category = "";$title = ""; $update = "";$match = 0;
}
if($count == 0){
		print "<p>マッチするメモテキストがありません。</p>\n";
}

# CGI 最終部分の出力
print "</table>\n";
print "<hr>\n";
print "[ <a href=\"$cgidir/wemo_list.cgi\">List</a> ]\n";
print " | [ <a href=\"$cgidir/wemo_calendar.cgi\">Calendar</a> ]\n";
# メモ作成・編集用フォームの CGI 出力
print "<FORM action=\"$cgidir/wemo_edit.cgi\">\n";
print "<INPUT TYPE=\"hidden\" NAME=\"today\" VALUE=\"yes\">\n";
# SELECTタグによるカテゴリの選択メニューを出力する
print "<SELECT name=\"category\" size=1>\n";
print "<OPTION selected></OPTION>\n";
# OPTIONタグで、カテゴリの選択メニューを作成する
foreach $catstr (sort keys(%categories)){
	print "<OPTION value=\"$catstr\">$categories{$catstr}</OPTION>\n";
}
print "</SELECT>\n";
# フォームの完成
print "<INPUT TYPE=\"submit\" VALUE=\"Edit Today\"></FORM>\n";
print "</BODY>\n</HTML>\n";
